"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Api = exports.ApiError = void 0;
const axios_1 = require("axios");
const debug = require("debug");
const FormData = require("form-data");
class ApiError extends Error {
}
exports.ApiError = ApiError;
class Api {
    constructor(token) {
        this.token = token;
        this.log = debug("tg-io:api");
        this.baseUrl = "https://api.telegram.org/bot";
    }
    /**
     * Call method with params as a formdata
     */
    async upload(method, params) {
        if (!(params instanceof FormData)) {
            let formdata = new FormData();
            for (let [key, value] of Object.entries(params))
                formdata.append(key, value);
            params = formdata;
        }
        return this.callMethod(method, params);
    }
    async callMethod(method, params) {
        var _a, _b, _c, _d, _e;
        let response;
        try {
            response = await axios_1.default.post(`${this.baseUrl}${this.token}/${method}`, params);
        }
        catch (reqError) {
            if (reqError instanceof axios_1.AxiosError) {
                let error = new ApiError((_b = (_a = reqError === null || reqError === void 0 ? void 0 : reqError.response) === null || _a === void 0 ? void 0 : _a.data) !== null && _b !== void 0 ? _b : reqError.message);
                error.code = (_e = (_d = (_c = reqError === null || reqError === void 0 ? void 0 : reqError.response) === null || _c === void 0 ? void 0 : _c.data) === null || _d === void 0 ? void 0 : _d.code) !== null && _e !== void 0 ? _e : reqError.code;
                error.method = method;
                error.params = JSON.stringify(params, null, 2);
                throw error;
            }
            throw reqError;
        }
        finally {
            this.log(`${method} ${response ? "OK" : "ERROR"}`);
        }
        return response.data;
    }
}
exports.Api = Api;
