"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageContext = void 0;
const attachments_1 = require("../models/attachments");
const Message_1 = require("../models/Message");
const Utils_1 = require("../Utils");
const ChatContext_1 = require("./ChatContext");
class MessageContext extends Message_1.Message {
    constructor(api, source) {
        super(source);
        this.api = api;
        this.match = [];
        this.chat = new ChatContext_1.ChatContext(this.api, this.get("chat"));
    }
    pin(id = this.id, disableNotification = true) {
        return this.api.callMethod("pinChatMessage", {
            chat_id: this.chat.id,
            message_id: id,
            disable_notification: disableNotification
        });
    }
    unpin(id = this.id) {
        return this.api.callMethod("unpinChatMessage", {
            chat_id: this.chat.id,
            message_id: id
        });
    }
    unpinAll() {
        return this.api.callMethod("unpinAllChatMessages", {
            chat_id: this.chat.id
        });
    }
    editText(text = this.text, params) {
        let isCaptionEdit = (params && params.message_id) || this.hasAttachments;
        params = Object.assign(Object.assign({ chat_id: this.chat.id }, (isCaptionEdit
            ? {
                caption: text
            }
            : { text })), (params !== null && params !== void 0 ? params : {}));
        return this.api.callMethod(isCaptionEdit
            ? "editMessageCaption"
            : "editMessageText", params);
    }
    delete(msgId = this.id) {
        let params = {
            message_id: msgId,
            chat_id: this.chat.id
        };
        return this.api.callMethod("deleteMessage", params);
    }
    replyMessage(params) {
        params = typeof params === "string"
            ? { text: params }
            : params;
        return this.reply("message", params);
    }
    sendMessage(params) {
        params = typeof params === "string"
            ? { text: params }
            : params;
        return this.send("message", params);
    }
    attach(method, params) {
        if (typeof params === "string")
            params = {
                [method]: (/^http/i.test(params)
                    ? params
                    : { file_id: params })
            };
        else if (params instanceof attachments_1.Attachment)
            params = {
                [method]: {
                    file_id: params.id
                }
            };
        else if (params.hasOwnProperty("file_id"))
            params = Object.assign({ [method]: {
                    file_id: params["file_id"],
                } }, (Utils_1.ObjectUtils.filterObjectByKey(params, k => k !== "file_id")));
        return this.send(method, params);
    }
    reply(method, params) {
        params.reply_to_message_id = this.id;
        return this.send(method, params);
    }
    async send(method, params) {
        params.chat_id = this.chat.id;
        let { result: response } = await this.api.callMethod(`send${Utils_1.StringUtils.capitalizeFirst(method)}`, params);
        return new Message_1.Message(response.message);
    }
}
exports.MessageContext = MessageContext;
